package se.liu.ida.InsuranceApp.services;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import se.liu.ida.InsuranceApp.data.CarData;
import se.liu.ida.InsuranceApp.data.ClientProfile;

import java.time.LocalDate;
import java.util.ArrayList;

@Service
public class InsuranceServiceImpl implements InsuranceService {

	@Autowired
	private ClientDataManagementService clientDB;
	private int counter;
    private int currYear = LocalDate.now().getYear();

	public InsuranceServiceImpl(ClientDataManagementService clientDB) {
		this.clientDB = clientDB;
		counter = 0;
	}

	@Override
	public Boolean isClientGoldMember(int clientId) {
		ClientProfile cl = clientDB.findById(clientId);
		return cl.getGoldMember();
	}
	
	@Override
	public Boolean registerNewMember(String lastName, String firstName, int yearOfBirth, int yearOfLicence) {
	    counter = clientDB.getCounterId();
		ClientProfile new_c = new ClientProfile(++counter, lastName, firstName, yearOfBirth, yearOfLicence);
		return clientDB.addClientProfile(new_c);
	}
	
	@Override
	public int registerNewAccident(int clientId) {
        ClientProfile cl = clientDB.findById(clientId);
        int accidents = cl.getNumberOfAccidentsThisYear() + 1;
        cl.setNumberOfAccidentsThisYear(accidents);
        clientDB.updateClientProfile(clientId, cl);
        return accidents;
	}

	@Override
	public Boolean updateDatabase() {
        for (int c = 1; c <= counter; c++)
        {
            ClientProfile cl = clientDB.findById(c);
            if (cl != null){
                if (cl.getNumberOfAccidentsThisYear() == 0 ){
                    cl.setYearsWithoutAccidents(cl.getYearsWithoutAccidents() + 1);
                    if (cl.getYearsWithoutAccidents() >= 5){
                        cl.setGoldMember(true);
                    }
                }else {
                    cl.setNumberOfAccidentsThisYear(0);
                }
            }
        }
		return null;
	}
	@Override
	public int getClientDeductible(int clientId) {
	    int baseCost = 0;
        ClientProfile cl = clientDB.findById(clientId);
        int currYear = LocalDate.now().getYear();
        if (currYear - cl.getYearOfBirth() > 30 || currYear - cl.getYearOfLicence() > 5 ){
            baseCost = 5000;
        }else{
            baseCost = 8000;
        }
        if (cl.getGoldMember() && cl.getNumberOfAccidentsThisYear() >= 3){
            switch (cl.getNumberOfAccidentsThisYear()){
                case 1:
                    baseCost += 1000;
                case 2:
                    baseCost += 2500;
                case 3:
                    baseCost += 4000;
                default:
                    baseCost += 10000;
            }
        }
		return baseCost;
	}
	@Override
	public int MonthlyInsuranceCost(int clientId) {
        int monthlyCost = 0;
        ClientProfile cl = clientDB.findById(clientId);

        if (currYear - cl.getYearOfBirth() > 30 || currYear - cl.getYearOfLicence() > 5 ){
            monthlyCost = 500;
        }else{
            monthlyCost = 600;
        }

        ArrayList<CarData> cars = cl.getCarsInsured();
        int count = 0;
        for (CarData car: cars) {
            if (count == 0){
                if (car.getCarColor() == "red"){
                    monthlyCost += 100;
                }
            } else {
                if (car.getCarColor() == "red"){
                    monthlyCost += 300;
                }else{
                    monthlyCost += 200;
                }
            }
            count++;
        }

        if ((cl.getGoldMember() || cl.getNumberOfAccidentsThisYear() == 0) && currYear - cl.getYearBecameClient() == 1){
            monthlyCost -= monthlyCost * 10 / 100;
        }
        return monthlyCost;
	}
	
	@Override
	public Boolean addNewCarToMember(int id, String color, int year) {
        ClientProfile cl = clientDB.findById(id);
        ArrayList<CarData> cars = cl.getCarsInsured();
        CarData newCar = new CarData(color, year);
        cars.add(newCar);
        cl.setCarsInsured(cars);
        clientDB.updateClientProfile(id, cl);
		return true;
	}

	@Override
	public ClientProfile getClientProfile(int id) {
        ClientProfile cl = clientDB.findById(id);
		return cl;
	}

}
