package se.liu.ida.InsuranceApp;


import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.extension.ExtendWith;
import org.mockito.Mock;
import org.mockito.junit.jupiter.MockitoExtension;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.web.servlet.AutoConfigureMockMvc;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.boot.test.mock.mockito.MockBean;
import org.springframework.test.context.ActiveProfiles;
import org.springframework.test.web.servlet.MockMvc;
import org.springframework.test.web.servlet.request.MockMvcRequestBuilders;
import org.springframework.test.web.servlet.setup.MockMvcBuilders;

import ch.qos.logback.core.status.Status;
import org.springframework.util.LinkedMultiValueMap;
import se.liu.ida.InsuranceApp.data.ClientProfile;
import se.liu.ida.InsuranceApp.services.ClientDataManagementService;
import se.liu.ida.InsuranceApp.services.ClientDataManagementServiceImpl;
import se.liu.ida.InsuranceApp.services.InsuranceService;
import se.liu.ida.InsuranceApp.services.InsuranceServiceImpl;

import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.status;


@SpringBootTest
@AutoConfigureMockMvc
@ExtendWith(MockitoExtension.class)
@ActiveProfiles("mockClientM")
class InsuranceAppApplicationMockMvcTests {

	@Autowired
	private MockDataTestConfig testConfig;

	@Autowired
	private MockMvc mvc;
	

	private InsuranceController ic;

	@Autowired
	ClientDataManagementService cm;

	InsuranceService is;
	
	@BeforeEach
    public void setup() {
		ic = new InsuranceController(new InsuranceServiceImpl(cm));
        this.mvc = MockMvcBuilders.standaloneSetup(ic).build();
        cm = testConfig.clientDataManagementService();
        is = testConfig.InsuranceService();
    }

	@Test
	public void test_addNewClient() throws Exception {
		LinkedMultiValueMap<String, String> requestParams = new LinkedMultiValueMap<>();
		requestParams.add("lastName", "Joan");
		requestParams.add("firstName", "Smith");
		requestParams.add("yearOfBirth", "1990");
		requestParams.add("yearOfLicence", "2013");
		mvc.perform(
				MockMvcRequestBuilders.get("/addNewClient").params(requestParams)).andExpect(status().isOk());
	}

	@Test
	public void test_addNewCar() throws Exception {
		LinkedMultiValueMap<String, String> requestParams = new LinkedMultiValueMap<>();
		requestParams.add("clientId", "1");
		requestParams.add("color", "Black");
		requestParams.add("year", "2019");
		mvc.perform(
				MockMvcRequestBuilders.get("/addNewCar").params(requestParams)).andExpect(status().isOk());
	}

	@Test
	public void test_getClientData() throws Exception {
		mvc.perform(
				MockMvcRequestBuilders.get("/getClientData").param("id", "1")).andExpect(status().isOk());
	}

	@Test
	public void test_getClientMonthlyRate() throws Exception {
		mvc.perform(
				MockMvcRequestBuilders.get("/getClientMonthlyRate").param("id", "1")).andExpect(status().isOk());
	}

	@Test
	public void test_getClientDeductible() throws Exception {
		mvc.perform(
				MockMvcRequestBuilders.get("/getClientDeductible").param("id", "1")).andExpect(status().isOk());
	}

	@Test
	public void test_addNewAccident() throws Exception {
		mvc.perform(
				MockMvcRequestBuilders.get("/addNewAccident").param("id", "1")).andExpect(status().isOk());
	}

	@Test
	public void test_registerNewMemberMock() throws Exception {
		assertEquals(true, is.registerNewMember("Joan", "ulf", 1980, 2000));
	}

	@Test
	public void test_isClientGoldMemberMock() throws Exception {
		assertEquals(false, is.isClientGoldMember(1));
	}

	@Test
	public void test_registerNewAccidentMock() throws Exception {
		assertEquals(1, is.registerNewAccident(1));
	}

	@Test
	public void test_MonthlyInsuranceCostMock() throws Exception {
		assertEquals(500, is.MonthlyInsuranceCost(1));
	}

	@Test
	public void test_getClientDeductibleMock() throws Exception {
		assertEquals(5000, is.getClientDeductible(1));
	}

	@Test
	public void test_addNewCarToMemberMock() throws Exception {
		assertEquals(true, is.addNewCarToMember(1, "Black", 2019));
	}

	@Test
	public void test_findByIdMock() throws Exception {
		ClientProfile t = cm.findById(1);
		boolean f = t.getLastName().equals("Joan");
		assertEquals(true, f);
	}

	@Test
	public void test_updateClientProfileMock() throws Exception {
		assertEquals(true, cm.updateClientProfile(1, null));
	}

	@Test
	public void test_addClientProfileMock() throws Exception {
		assertEquals(true, cm.addClientProfile(null));
	}

	@Test
	public void test_removeClientProfileMock() throws Exception {
		assertEquals(true, cm.removeClientProfile(1));
	}
}
