package se.liu.ida.InsuranceApp;

import static org.junit.jupiter.api.Assertions.assertArrayEquals;
import static org.junit.jupiter.api.Assertions.assertEquals;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.extension.ExtendWith;
import org.mockito.junit.jupiter.MockitoExtension;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.test.context.ActiveProfiles;

import se.liu.ida.InsuranceApp.data.ClientProfile;
import se.liu.ida.InsuranceApp.services.ClientDataManagementService;
import se.liu.ida.InsuranceApp.services.ClientDataManagementServiceImpl;
import se.liu.ida.InsuranceApp.services.InsuranceService;
import se.liu.ida.InsuranceApp.services.InsuranceServiceImpl;


@SpringBootTest
@ExtendWith(MockitoExtension.class)
@ActiveProfiles("mockClientM")
class InsuranceAppApplicationTests {

	@Autowired
	ClientDataManagementService cm;
	
	// the service we are testing
	@Autowired
	InsuranceService is;

	
	@BeforeEach
    public void init() {
		cm = new ClientDataManagementServiceImpl();
		is = new InsuranceServiceImpl(cm);
    }
	
	 @Test
	 public void test_newClient_notGoldMember() throws Exception {
	       assertEquals(false, is.isClientGoldMember(1));
	 }

	@Test
	public void test_registerNewMember() throws Exception {
		assertEquals(true, is.registerNewMember("Joan", "Smith", 1990, 2013));
	}

	@Test
	public void test_registerNewMember1() throws Exception {
		assertEquals(true, is.registerNewMember("abc", "xyz", 1991, 2012));
	}

	@Test
	public void test_addNewCarToMember() throws Exception {
		assertEquals(true, is.addNewCarToMember(1, "Black", 2015));
	}

	@Test
	public void test_getClientProfile() throws Exception {
		assertEquals("Smith", is.getClientProfile(1).getFirstName());
	}

	@Test
	public void test_getClientDeductible() throws Exception {
		assertEquals(5000, is.getClientDeductible(1));
	}

	@Test
	public void test_registerNewAccident() throws Exception {
		assertEquals(1, is.registerNewAccident(1));
	}

	@Test
	public void test_MonthlyInsuranceCost() throws Exception {
		assertEquals(500, is.MonthlyInsuranceCost(1));
	}

	@Test
	public void test_findById() throws Exception {
		assertEquals(null, cm.findById(11));
	}

	@Test
	public void test_addClientProfile() throws Exception {
		assertEquals(true, cm.addClientProfile(new ClientProfile(12, "Jo", "Smith", 1980, 2000)));
	}

	@Test
	public void test_removeClientProfile() throws Exception {
		assertEquals(true, cm.removeClientProfile(2));
	}

	@Test
	public void test_updateClientProfile() throws Exception {
		assertEquals(true, cm.updateClientProfile(2, new ClientProfile(12, "Jo", "Smith", 1980, 2000)));
	}

}
